/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.ServiceProcess;
using NAnt.Core.Types;
using NAnt.Core.Tasks;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// <p>
    /// Extends <c>ApplicationServer</c> abstract class and represents a web application server to be used while
    /// running tests. This class supports the Microsoft IIS server. And it should support almost any
    /// service-based servers.
    /// </p>
    /// <p>
    /// The supported options are:
    /// SERVICE_NAME - specifies the name of service of the server.
    /// </p>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe since its parent class is mutable.
    /// </remarks>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class IISServer : ApplicationServer
    {
        /// <summary>
        /// Represents the name(key) of the service_name option. The value of the service_name option
        /// represents the name of service of the server. This option is not required.
        /// The DEFAULT_SERVICE_NAME constant is used as a default value of option.
        /// </summary>
        public static readonly string SERVICE_NAME = "service_name";

        /// <summary>
        /// Represents default value for the SERVICE_NAME option. It the name of IIS service.
        /// </summary>
        private static readonly string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// Represents default value for the TIMEOUT option. It is represented in milliseconds.
        /// </summary>
        private static readonly int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// Creates a new instance of <c>IISServer</c> class.
        /// </summary>
        public IISServer() : base()
        {
        }

        /// <summary>
        /// Starts the server. Before starting the server, the web-application should be deployed first to a
        /// specific directory according the pre-defined option value if the 'initserver' attribute is
        /// <c>true</c>..
        /// </summary>
        /// <remarks>
        /// In the case when 'startserver' attribute is set to false, the server should not be started.
        /// </remarks>
        /// <exception cref="ServerException">If it fails to start the server or fail to deploy the
        /// web-application.</exception>
        public override void StartUp()
        {
            // 1. deploy the application
            if (InitServer)
            {
                Deploy();
            }

            // 2. starts the server
            try
            {
                // Get the server service name
                string serviceName = GetOption(SERVICE_NAME);
                serviceName = serviceName == null ? DEFAULT_SERVICE_NAME : serviceName;

                // Create an appropriate ServiceController
                ServiceController serviceController = new ServiceController(serviceName);

                // Get timeout
                string timeoutString = GetOption(TIMEOUT);
                int timeout = timeoutString == null ? DEFAULT_TIMEOUT : GetNonNegetiveIntValue(timeoutString);

                // Check if the service was is running
                if (serviceController.Status == ServiceControllerStatus.Running)
                {
                    // If it is running, the ServerException should be thrown
                    throw new ServerException("The service has already started.");
                }

                // Start the service
                if (serviceController.Status == ServiceControllerStatus.Paused)
                {
                    serviceController.Continue();
                }
                else if (serviceController.Status == ServiceControllerStatus.Stopped)
                {
                    serviceController.Start();
                }
                else
                {
                    throw new ServerException("The service status of IIS is illegal.");
                }

                // Wait for the service to reach the desired status, or the timeout expired
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(timeout));
            }
            catch (ServerException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                throw new ServerException("Fails to start the server.", e);
            }
        }

        /// <summary>
        /// Deploies the web-application.
        /// </summary>
        /// <exception cref="ServerException">If it fails to deploy the web-application.</exception>
        private void Deploy()
        {
            try
            {
                // Create a NAnt copy task
                CopyTask copyTask = new CopyTask();

                // Specify destination directory
                copyTask.ToDirectory = DeploymentDirectory;
                Console.WriteLine(DeploymentDirectory);

                // Specify files to copy
                copyTask.CopyFileSet = SourceFiles;

                // Specify that files should be overwritten
                copyTask.Overwrite = true;

                // Set the project
                copyTask.Project = Project;

                // Execute the task
                copyTask.Execute();
            }
            catch (Exception e)
            {
                throw new ServerException("Fails to deploy the web-application.", e);
            }
        }

        /// <summary>
        /// Gets the option value from the options dictionary with the given option name. If the option value is
        /// missing or is just an empty string, <c>null</c> will be returned.
        /// </summary>
        /// <param name="optionName">The option name to retrieve the option value.</param>
        /// <returns>The retrieved option value.</returns>
        private string GetOption(string optionName)
        {
            Option option = Options[optionName];
            if (option == null || option.Value.Trim().Length == 0)
            {
                return null;
            }
            return option.Value;
        }

        /// <summary>
        /// Parses a non-negetive integer from the given string value.
        /// </summary>
        /// <param name="stringValue">The given string value where the non-negetive integer is retrieved.</param>
        /// <returns>A non-negetive integer from the given string value.</returns>
        /// <exception cref="ServerException">The string value is not a non-negative integer.</exception>
        private int GetNonNegetiveIntValue(string stringValue)
        {
            int value;

            try
            {
                value = Int32.Parse(stringValue);
            }
            catch (FormatException e)
            {
                throw new ServerException(stringValue + " is invalid.", e);
            }
            catch (OverflowException e)
            {
                throw new ServerException(stringValue + " is invalid.", e);
            }

            // Check non-negetive
            if (value < 0)
            {
                throw new ServerException(stringValue + " must be non-negative.");
            }

            return value;
        }

        /// <summary>
        /// Stops the server. Web-application should be deleted if the 'initserver' attribute is <c>true</c>.
        /// </summary>
        /// <remarks>
        /// The server should not be stopped if it wasn't started by the <c>startUp()</c> method.
        /// </remarks>
        /// <exception cref="ServerException">If it fails to shutdown the server or fail to delete the
        /// web-application.</exception>
        public override void ShutDown()
        {
            // 1. stops the server
            try
            {
                // Get the server service name
                string serviceName = GetOption(SERVICE_NAME);
                serviceName = serviceName == null ? DEFAULT_SERVICE_NAME : serviceName;

                // Create an appropriate ServiceController
                ServiceController serviceController = new ServiceController(serviceName);

                // Get timeout
                string timeoutString = GetOption(TIMEOUT);
                int timeout = timeoutString == null ? DEFAULT_TIMEOUT : GetNonNegetiveIntValue(timeoutString);

                // Stop the service
                if (serviceController.CanStop)
                {
                    serviceController.Stop();
                }
                else
                {
                    throw new ServerException("The service can not be stopped.");
                }

                // Wait for the service to reach the desired status, or the timeout expired
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(timeout));
            }
            catch (ServerException e)
            {
                throw e;
            }
            catch (Exception e)
            {
                throw new ServerException("Fails to stop the server.", e);
            }

            // After the server is stopped, should delete the web-application
            if (InitServer) {
                UnDeploy();
            }
        }

        /// <summary>
        /// Deletes the web-application.
        /// </summary>
        /// <exception cref="ServerException">If it fails to delete the web-application.</exception>
        private void UnDeploy()
        {
            try
            {
                // Create NAnt file delete task
                DeleteTask deleteTask = new DeleteTask();

                // Specify the files to delete
                FileSet deployFileSet = new FileSet();
                deployFileSet.BaseDirectory = DeploymentDirectory;
                FileSet.IncludesElement[] includes = new FileSet.IncludesElement[SourceFiles.Includes.Count];
                for (int i = 0; i < SourceFiles.Includes.Count; i++)
                {
                    includes[i] = new FileSet.IncludesElement();
                    includes[i].Pattern = SourceFiles.Includes[i];
                }
                deployFileSet.SetIncludes = includes;
                FileSet.ExcludesElement[] excludes = new FileSet.ExcludesElement[SourceFiles.Excludes.Count];
                for (int i = 0; i < SourceFiles.Excludes.Count; i++)
                {
                    excludes[i] = new FileSet.ExcludesElement();
                    excludes[i].Pattern = SourceFiles.Excludes[i];
                }
                deployFileSet.SetExcludes = excludes;
                deployFileSet.Project = SourceFiles.Project;
                deleteTask.DeleteFileSet = deployFileSet;

                // Set the project
                deleteTask.Project = Project;

                // Execute the task
                deleteTask.Execute();
            }
            catch (Exception e)
            {
                throw new ServerException("Fails to delete the web-application.", e);
            }
        }
        /// <summary>
        /// Initializes the server with data.
        /// </summary>
        /// <remarks>
        /// In the case when 'initserver' attribute is set to false, does nothing, or can check if the data
        /// are properly initialized,
        /// </remarks>
        public override void InitData()
        {
            // do nothing since the application is deployed in StartUp() method
        }

        /// <summary>
        /// Clean-ups the data initialized by the <c>initData()</c> method.
        /// </summary>
        /// <remarks>
        /// In the case when 'initserver' attribute is set to false or data was not initialized, does nothing.
        /// </remarks>
        public override void CleanUpData()
        {
            // do nothing since the application is removed from deployment dir in ShutDown() method
        }
    }
}