/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Tests the functionality and error cases of <c>Server</c> class.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public abstract class ServerUnitTest
    {
        /// <summary>
        /// A type reference to the current <see cref="Server"/> subclass for testing. Set in the constructors
        /// for each subclass.
        /// </summary>
        protected Type meType;

        /// <summary>
        /// Represents the <c>Server</c> instance used in tests.
        /// </summary>
        protected Server server = null;

        /// <summary>
        /// Sets up the test environment. The instances for testing will be created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            server = Constructor(new Type[0], new object[0]);
        }

        /// <summary>
        /// Calls the constructor for the specific type, with parameters.
        /// </summary>
        /// <param name="types">A list of the argument types.</param>
        /// <param name="paramList">The parameter values.</param>
        /// <returns>The created instance with given type and parameters.</returns>
        public Server Constructor(Type[] types, object[] paramList)
        {
            return (Server) meType.GetConstructor(types).Invoke(paramList);
        }

        /// <summary>
        /// Tests the accuracy of the constructor <c>Server</c>.
        /// </summary>
        [Test]
        public void TestServer_Accuracy()
        {
            Assert.IsNotNull(server, "Constructor should be correct.");
        }

        /// <summary>
        /// Tests the accuracy of <c>StartServer</c> Property.
        /// </summary>
        [Test]
        public void TestStartServer_Accuracy()
        {
            Assert.AreEqual(true, server.StartServer, "The startServer field should be got properly.");
            server.StartServer = false;
            Assert.AreEqual(false, server.StartServer, "The startServer field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>InitServer</c> Property.
        /// </summary>
        [Test]
        public void TestInitServer_Accuracy()
        {
            Assert.AreEqual(true, server.InitServer, "The initServer field should be got properly.");
            server.InitServer = false;
            Assert.AreEqual(false, server.InitServer, "The initServer field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Options</c> Property.
        /// </summary>
        [Test]
        public void TestOptions_Accuracy()
        {
            Assert.AreEqual(null, server.Options, "The options field should be got properly.");
            OptionCollection collection = new OptionCollection();
            server.Options = collection;
            Assert.AreEqual(collection, server.Options, "The options field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Credentials</c> Property.
        /// </summary>
        [Test]
        public void TestCredentials_Accuracy()
        {
            Assert.AreEqual(null, server.Credentials, "The credentials field should be got properly.");
            Credential credential = new Credential();
            server.Credentials = credential;
            Assert.AreEqual(credential, server.Credentials, "The credentials field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>SourceFiles</c> Property.
        /// </summary>
        [Test]
        public void TestSourceFiles_Accuracy()
        {
            Assert.AreEqual(null, server.SourceFiles, "The sourceFiles field should be got properly.");
            FileSet fileSet = new FileSet();
            server.SourceFiles = fileSet;
            Assert.AreEqual(fileSet, server.SourceFiles, "The sourceFiles field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Project</c> Property.
        /// </summary>
        [Test]
        public void TestProject_Accuracy()
        {
            Assert.AreEqual(null, server.Project, "The project field should be got properly.");
            Project project = new Project(@"../../default.build", Level.Info);
            server.Project = project;
            Assert.AreEqual(project, server.Project, "The project field should be set properly.");
        }

        /// <summary>
        /// Tests the <c>Timeout</c> Property when the option value is missing, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeout_MissingOptionValue()
        {
            server.Options = new OptionCollection();
            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests the <c>Timeout</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeout_InvalidOptionValue1()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.TIMEOUT }, new string[] { "-1" });
            server.Options = collection;

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests the <c>Timeout</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeout_InvalidOptionValue2()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.TIMEOUT }, new string[] { "ab" });
            server.Options = collection;

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests the <c>Timeout</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestTimeout_InvalidOptionValue3()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.TIMEOUT }, new string[] { "11111111111111111111111111111111111" });
            server.Options = collection;

            int timeout = server.Timeout;
        }

        /// <summary>
        /// Tests the accuracy of <c>Timeout</c> Property.
        /// </summary>
        [Test]
        public void TestTimeout_GetAccuracy()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.TIMEOUT }, new string[] { "1" });
            server.Options = collection;

            // check the value
            Assert.AreEqual(1, server.Timeout, "The value of timeout should be got properly.");
        }

        /// <summary>
        /// Tests the <c>Port</c> Property when the option value is missing, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPort_MissingOptionValue()
        {
            server.Options = new OptionCollection();
            int port = server.Port;
        }

        /// <summary>
        /// Tests the <c>Port</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPort_InvalidOptionValue1()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.PORT }, new string[] { "-1" });
            server.Options = collection;

            int port = server.Port;
        }

        /// <summary>
        /// Tests the <c>Port</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPort_InvalidOptionValue2()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.PORT }, new string[] { "100000" });
            server.Options = collection;

            int port = server.Port;
        }

        /// <summary>
        /// Tests the <c>Port</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPort_InvalidOptionValue3()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.PORT }, new string[] { "ab1" });
            server.Options = collection;

            int port = server.Port;
        }

        /// <summary>
        /// Tests the <c>Port</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestPort_InvalidOptionValue4()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.PORT }, new string[] { "1111111111111111111111111111111111111111" });
            server.Options = collection;

            int port = server.Port;
        }

        /// <summary>
        /// Tests the accuracy of <c>Port</c> Property.
        /// </summary>
        [Test]
        public void TestPort_GetAccuracy()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.PORT }, new string[] { "1" });
            server.Options = collection;

            // check the value
            Assert.AreEqual(1, server.Port, "The value of port should be got properly.");
        }

        /// <summary>
        /// Tests the <c>Address</c> Property when the option value is missing, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestAddress_MissingOptionValue()
        {
            server.Options = new OptionCollection();
            string address = server.Address;
        }

        /// <summary>
        /// Tests the <c>Address</c> Property when the option value is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestAddress_InvalidOptionValue()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.ADDRESS }, new string[] { " " });
            server.Options = collection;

            string address = server.Address;
        }

        /// <summary>
        /// Tests the accuracy of <c>Address</c> Property.
        /// </summary>
        [Test]
        public void TestAddress_GetAccuracy()
        {
            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { Server.ADDRESS }, new string[] { "address" });
            server.Options = collection;

            // check the value
            Assert.AreEqual("address", server.Address, "The value of address should be got properly.");
        }
    }
}
